import os
import json
import time
import requests
import traceback
import code2api

import code2api.exceptions as exceptions
from code2api.loginInstance import login_instance
import code2api.configs as configs


def manipulation_detection(image_path, version='', max_interval=configs.max_interval):
	# default version:
	algorithm_name = 'manipulation_detection'
	algorithm_version = 'v1.0'
	algorithm_port = 17789
	algorithm_ip = '120.26.143.61'

	full_algorithm_name = algorithm_name+'-'+algorithm_version
	lower_name = full_algorithm_name.replace('-', '_').replace('.', '_').lower()
	upper_name = full_algorithm_name.replace('-', '_').replace('.', '_').upper()

	upload_url = 'http://{}/upload_files/manipulation_detection/1_0_0'.format(algorithm_ip)
	filename = os.path.split(image_path)[-1]
	with open(image_path, 'rb') as f:
		files = {'file': (filename, f.read())}
	response = requests.post(upload_url, files=files)
	param_path = json.loads(response.content)['return_path']
	params = {
		'input_path': param_path
	}

	submit_url = 'http://{}:{}/{}_submit'.format(algorithm_ip, algorithm_port, lower_name)
	result_url = 'http://{}:{}//{}/get_result'.format(algorithm_ip, algorithm_port, lower_name)
	headers = {'Cookie': login_instance.cookie}
	response = requests.post(submit_url, data=params, headers=headers)
	if response.status_code == 403:
		raise exceptions.LoginError('[VSOURCE-Lib] 登陆信息失败，请先正常的登陆')
	response_dict = json.loads(response.content)
	task_id = response_dict['id']
	start_time = time.time()
	while True:
		# 轮询查询结果
		if time.time()-start_time>=max_interval:
			raise exceptions.TimeOutException('[VSOURCE-Lib] 请求超时，未知错误')
		face_result = requests.get(result_url, params={'id': task_id}, headers=headers, timeout=max_interval)
		face_ans = json.loads(face_result.content)
		if face_ans['status'] != 200:
			time.sleep(configs.interval)
			continue
		assert face_ans['status'] == 200
		if face_ans['result']['status'] == 'error':
			return -1
		image_path = face_ans['result']['result']['output_path']
		score = face_ans['result']['result']['score']
		final_image_path = 'http://{}/get_files/'.format(algorithm_ip)+image_path

		out = {
			'output_path': final_image_path,
			'score': score,
		}
		return out
	raise exceptions.TimeOutException('[VSOURCE-Lib] 请求超时，未知错误')




# -*- coding: utf-8 -*-
# @Author   : Ecohnoch(xcy)
# @File     : new_algorithm.py
# @Function : TODO


import json
import time
import requests

import code2api.exceptions as exceptions
from code2api.loginInstance import login_instance
import code2api.configs as configs

def github_avatar_generation(name, version='none', max_interval=configs.max_interval):
    algorithm_name = 'github_avatar_generator'
    algorithm_version = 'v1.1'
    algorithm_port = 17785
    algorithm_ip   = '120.26.143.61'
    full_algorithm_name = algorithm_name + '-' + algorithm_version
    lower_name = full_algorithm_name.replace('-', '_').replace('.', '_').lower()
    upper_name = full_algorithm_name.replace('-', '_').replace('.', '_').upper()
    submit_url = 'http://{}:{}/{}_submit'.format(algorithm_ip, algorithm_port, lower_name)
    result_url = 'http://{}:{}//{}/get_result'.format(algorithm_ip, algorithm_port, lower_name)
    headers = {'Cookie': login_instance.cookie}
    post_params = {
        'name': name,
    }
    response = requests.post(submit_url, data=post_params, headers=headers)
    if response.status_code == 403:
        raise exceptions.LoginError('[VSOURCE-Lib] 登陆信息失败，请先正常的登陆')
    response_dict = json.loads(response.content)
    task_id = response_dict['id']
    start_time = time.time()
    while True:
        # 轮询查询结果
        if time.time() - start_time >= max_interval:
            raise exceptions.TimeOutException('[VSOURCE-Lib] 请求超时，未知错误')
        face_result = requests.get(result_url, params={'id': task_id}, headers=headers, timeout=max_interval)
        face_ans = json.loads(face_result.content)
        if face_ans['status'] != 200:
            time.sleep(configs.interval)
            continue
        assert face_ans['status'] == 200
        if face_ans['result']['status'] == 'error':
            return -1
        image_path = face_ans['result']['result']['result']
        final_image_path = 'http://{}/get_files/'.format(algorithm_ip) + image_path
        return final_image_path
    raise exceptions.TimeOutException('[VSOURCE-Lib] 请求超时，未知错误')


import vsource

if __name__ == '__main__':
    algorithm_config = {
        # 算法定义的名字
        'algorithm-name': 'random_number',
        # 算法的版本号
        'version': '1.0.2',
        # 算法的作者
        'author': 'xcy',
        # 算法工程文件夹的路径
        'service-dir': './',
        # 算法函数的所处的Python文件名
        'service-filename': 'random_number.py',
        # 算法的函数的标识符
        'service-interface': 'random_number',
        # 算法所依赖的Python库及版本
        'requirements': [
            'numpy==1.19.0',
        ],
        # 算法所依赖的非Python的软件，以Ubuntu的安装方式为标准
        'pre-command': [
            # 'apt install -y libgl1-mesa-glx'
        ],
        'input-params': {
            'lower': {
                'type': 'float',
                'describe': 'lower limit for thin function'
            },
            'upper': {
                'type': 'float',
                'describe': 'upper limit for thin function'
            },
        },
        'output-params': {
            'result': {
                'type': 'float',
                'describe': 'calculated  result'
            }
        }
    }

    vsource.parse_algorithm(algorithm_config)


import code2api.configs as configs

def face_attribute(face_path, max_interval=configs.max_interval, version='none'):
    if version == 'fsx':
        from code2api.algorithms.face_attribute_x import face_attribute_x
        return face_attribute_x(face_path, version='fsx-0.1', max_interval=max_interval)
    if version == 'cgy-2':
        from code2api.algorithms.face_attribute_x import face_attribute_y
        return face_attribute_y(face_path, version='cgy-2', max_interval=max_interval)
    from code2api.algorithms.face_attribute_x import face_attribute_z
    return face_attribute_z(face_path, version='cgy-1', max_interval=max_interval)


# -*- coding: utf-8 -*-
# @Author   : Ecohnoch(xcy)
# @File     : __init__.py.py
# @Function : TODO

from code2api.loginInstance import login
from code2api.algorithms.face_recognition import face_recognition
from code2api.algorithms.face_detection import face_detection
from code2api.algorithms.face_attribute import face_attribute
from code2api.algorithms.speaker_recognition import speaker_recognition
from code2api.algorithms.face_generation import face_generation
from code2api.algorithms.random_number import random_number
from code2api.algorithms.remove_bg import remove_bg

from code2api.algorithms.new_algorithm import new_algorithm
from code2api.algorithms.cod_cpd import cod_cpd
from code2api.algorithms.salient_object_detection import salient_object_detection
from code2api.algorithms.landmark_detection import landmark_detection
from code2api.algorithms.binary_segmentation import binary_segmentation
from code2api.algorithms.face_attribute_x import face_attribute_x
from code2api.algorithms.text_generation import text_generation
from code2api.algorithms.github_avatar_generation import github_avatar_generation
from code2api.algorithms.anime_gan_v2 import anime_generation
from code2api.algorithms.maze_generation import maze_generation
from code2api.algorithms.hrnet_landmark_detection import hrnet_landmark_detection
from code2api.algorithms.textbox import textbox_generate_sentence
from code2api.algorithms.sod import sod


from code2api.scripts.algorithm_scripts.algorithms import parse_algorithm
from code2api.scripts.enroll_service import enroll

__version__ = 'v1.1.5'
__author__  = 'Ecohnoch'
__email__   = 'chuyuan@code2api.club'